/***************************************************************************
 *   Copyright (C) 2008 by S. MANKOWSKI / G. DE BURE skrooge@miraks.com    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>  *
 ***************************************************************************/
/** @file
 * This file is part of Skrooge and defines classes SKGRuleObject.
 *
 * @author Stephane MANKOWSKI / Guillaume DE BURE
 */
#include "skgruleobject.h"
#include "skgunitvalueobject.h"
#include "skgtraces.h"

#include <klocale.h>

#include <QDomDocument>

SKGRuleObject::SKGRuleObject(const SKGDocument* iDocument, int iID)
                : SKGObjectBase(iDocument, "v_rule", iID)
{
}

SKGRuleObject::SKGRuleObject(const SKGRuleObject& iObject)
                : SKGObjectBase(iObject)
{
}

SKGRuleObject::SKGRuleObject(const SKGObjectBase& iObject)
{
        if (iObject.getRealTable()=="rule") {
                copyFrom(iObject);
        } else {
                *this=SKGObjectBase::SKGObjectBase(iObject.getDocument(), "v_rule", iObject.getID());
        }
}

const SKGRuleObject& SKGRuleObject::operator= (const SKGObjectBase& iObject)
{
        copyFrom(iObject);
        return *this;
}

SKGRuleObject::~SKGRuleObject()
{
}

SKGError SKGRuleObject::setXMLSearchDefinition(const QString& iXml)
{
        setSearchDescription(SKGRuleObject::getDescriptionFromXML((SKGDocument*) getDocument(), "operation", iXml, false, 0));
        return setAttribute("t_definition", iXml);
}

QString SKGRuleObject::getXMLSearchDefinition() const
{
        return getAttribute("t_definition");
}

SKGError SKGRuleObject::setXMLActionDefinition(const QString& iXml)
{
        setActionDescription(SKGRuleObject::getDescriptionFromXML((SKGDocument*) getDocument(), "operation", iXml, false, 1));
        return setAttribute("t_action_definition", iXml);
}

QString SKGRuleObject::getXMLActionDefinition() const
{
        return getAttribute("t_action_definition");
}

SKGError SKGRuleObject::setSearchDescription(const QString& iDescription)
{
        return setAttribute("t_description", iDescription);
}

QString SKGRuleObject::getSearchDescription() const
{
        return getAttribute("t_description");
}

SKGError SKGRuleObject::setActionDescription(const QString& iDescription)
{
        return setAttribute("t_action_description", iDescription);
}

QString SKGRuleObject::getActionDescription() const
{
        return getAttribute("t_action_description");
}

SKGError SKGRuleObject::setOrder(double iOrder)
{
        SKGError err;
        double order=iOrder;
        if (order==-1) {
                order=1;
                SKGStringListList result;
                err=SKGServices::executeSelectSqliteOrder(getDocument(), "SELECT max(f_sortorder) from rule", result);
                if (err.isSucceeded() && result.count()==2) order=SKGServices::stringToDouble(result.at(1).at(0))+1;
        }
        if (err.isSucceeded()) err=setAttribute("f_sortorder", SKGServices::doubleToString(order));
        return err;
}

QString SKGRuleObject::getSelectSqlOrder(const QString& iAdditionalCondition) const
{
        QString wc=iAdditionalCondition;
        QString wc2=SKGRuleObject::getDescriptionFromXML((SKGDocument*) getDocument(), "operation", getXMLSearchDefinition(), true, 0);
        if (!wc2.isEmpty()) {
                if (wc.isEmpty()) wc=wc2;
                else wc='('+wc+") AND ("+wc2+')';
        }
        if (wc.isEmpty()) wc="1=1";
        return wc;
}

QString SKGRuleObject::getUpdateSqlOrder(const QString& iTable, const QString& iAdditionalCondition) const
{
        QString output;
        QStringList list=SKGRuleObject::getFromXML((SKGDocument*) getDocument(), "operation", getXMLActionDefinition(), true, 2); //SQL + SET clause
        int nb=list.count();
        for (int i=0; i<nb; ++i) {
                output.append(list.at(i));
                if (i<nb-1) output.append(";;");
        }

        QString wc=getSelectSqlOrder(iAdditionalCondition);
        if (wc.isEmpty()) wc="1=1";
        output.replace("#WC#", wc);

        return output;
}

double SKGRuleObject::getOrder() const
{
        return SKGServices::stringToDouble(getAttribute("f_sortorder"));
}

QString SKGRuleObject::getDisplayForOperator(const QString& iOperator, const QString& iParam1, const QString& iParam2)
{
        QString output=iOperator;
        if (output=="%9 LIKE '%%1%'") output=i18n("contains '%1'", iParam1);
        else if (output=="%9 LIKE '%1%'") output=i18n("starts with '%1'", iParam1);
        else if (output=="%9 LIKE '%%1'") output=i18n("ends with '%1'", iParam1);
        else if (output=="%9=''") output=i18n("is empty");
        else if (output=="%9!=''") output=i18n("is not empty");

        else if (output=="%9=%1") output=i18n("=%1", iParam1);
        else if (output=="%9!=%1") output=i18n("!=%1", iParam1);
        else if (output=="%9>%1") output=i18n(">%1", iParam1);
        else if (output=="%9<%1") output=i18n("<%1", iParam1);
        else if (output=="%9>=%1") output=i18n(">=%1", iParam1);
        else if (output=="%9<=%1") output=i18n("<=%1", iParam1);

        else if (output=="%9='%1'") output=i18n("='%1'", iParam1);
        else if (output=="%9!='%1'") output=i18n("!='%1'", iParam1);
        else if (output=="%9>'%1'") output=i18n(">'%1'", iParam1);
        else if (output=="%9<'%1'") output=i18n("<'%1'", iParam1);
        else if (output=="%9>='%1'") output=i18n(">='%1'", iParam1);
        else if (output=="%9<='%1'") output=i18n("<='%1'", iParam1);

        else if (output=="%9>=%1 AND %9<=%2") output=i18n("is between %1 and %2", iParam1, iParam2);
        else if (output=="%9>='%1' AND %9<='%2'") output=i18n("is between '%1' and '%2'", iParam1, iParam2);

        else if (output=="%9=lower(%9)") output=i18n("is set to lower");
        else if (output=="%9=upper(%9)") output=i18n("is set to upper");

        else if (output=="%9!=lower(%9)") output=i18n("is not lower");
        else if (output=="%9!=upper(%9)") output=i18n("is not upper");

        else if (output=="STRFTIME('%Y-%m',%9)=STRFTIME('%Y-%m',date('now'))") output=i18n("is in current month");
        else if (output=="STRFTIME('%Y-%m',%9)=STRFTIME('%Y-%m',date('now','-1 month'))") output=i18n("is in previous month");
        else if (output=="STRFTIME('%Y',%9)=STRFTIME('%Y',date('now'))") output=i18n("is in current year");
        else if (output=="STRFTIME('%Y',%9)=STRFTIME('%Y',date('now','-1 year'))") output=i18n("is in previous year");
        else if (output=="%9>date('now','-30 day') AND %9<=date('now')") output=i18n("is in last 30 days");
        else if (output=="%9>date('now','-3 month') AND %9<=date('now')") output=i18n("is in last 3 months");
        else if (output=="%9>date('now','-6 month') AND %9<=date('now')") output=i18n("is in last 6 months");
        else if (output=="%9>date('now','-12 month') AND %9<=date('now')") output=i18n("is in last 12 months");
        else if (output=="%9>date('now','-2 year') AND %9<=date('now')") output=i18n("is in last 2 years");
        else if (output=="%9>date('now','-3 year') AND %9<=date('now')") output=i18n("is in last 3 years");
        else if (output=="%9>date('now','-5 year') AND %9<=date('now')") output=i18n("is in last 5 years");
        return output;
}

QStringList SKGRuleObject::getListOfOperators(SKGServices::AttibuteType iAttributeType, bool iUpdateMode)
{
        QStringList output;
        if (iUpdateMode) {
                //Mode update
                if (iAttributeType==SKGServices::TEXT) {
                        output.push_back("%9='%1'");
                        output.push_back("%9=lower(%9)");
                        output.push_back("%9=upper(%9)");
                } else if (iAttributeType==SKGServices::INTEGER || iAttributeType==SKGServices::FLOAT) {
                        output.push_back("%9=%1");
                } else if (iAttributeType==SKGServices::DATE || iAttributeType==SKGServices::BOOL || iAttributeType==SKGServices::TRISTATE) {
                        output.push_back("%9='%1'");
                }

        } else {
                //Mode query
                if (iAttributeType==SKGServices::TEXT) {
                        output.push_back("%9 LIKE '%%1%'");
                        output.push_back("%9 LIKE '%1%'");
                        output.push_back("%9 LIKE '%%1'");
                        output.push_back("%9=''");
                        output.push_back("%9!=''");
                        output.push_back("%9!=lower(%9)");
                        output.push_back("%9!=upper(%9)");
                }

                if (iAttributeType==SKGServices::INTEGER || iAttributeType==SKGServices::FLOAT) {
                        output.push_back("%9=%1");
                        output.push_back("%9!=%1");
                        output.push_back("%9>%1");
                        output.push_back("%9<%1");
                        output.push_back("%9>=%1");
                        output.push_back("%9<=%1");
                        output.push_back("%9>=%1 AND %9<=%2");
                }

                if (iAttributeType==SKGServices::BOOL || iAttributeType==SKGServices::TRISTATE || iAttributeType==SKGServices::TEXT || iAttributeType==SKGServices::DATE) {
                        output.push_back("%9='%1'");
                }

                if (iAttributeType==SKGServices::TRISTATE || iAttributeType==SKGServices::TEXT || iAttributeType==SKGServices::DATE) {
                        output.push_back("%9!='%1'");
                }

                if (iAttributeType==SKGServices::TEXT || iAttributeType==SKGServices::DATE) {
                        output.push_back("%9>'%1'");
                        output.push_back("%9<'%1'");
                        output.push_back("%9>='%1'");
                        output.push_back("%9<='%1'");
                        output.push_back("%9>='%1' AND %9<='%2'");
                }

                if (iAttributeType==SKGServices::DATE) {
                        output.push_back("STRFTIME('%Y-%m',%9)=STRFTIME('%Y-%m',date('now'))");
                        output.push_back("STRFTIME('%Y-%m',%9)=STRFTIME('%Y-%m',date('now','-1 month'))");
                        output.push_back("STRFTIME('%Y',%9)=STRFTIME('%Y',date('now'))");
                        output.push_back("STRFTIME('%Y',%9)=STRFTIME('%Y',date('now','-1 year'))");
                        output.push_back("%9>date('now','-30 day') AND %9<=date('now')");
                        output.push_back("%9>date('now','-3 month') AND %9<=date('now')");
                        output.push_back("%9>date('now','-6 month') AND %9<=date('now')");
                        output.push_back("%9>date('now','-12 month') AND %9<=date('now')");
                        output.push_back("%9>date('now','-2 year') AND %9<=date('now')");
                        output.push_back("%9>date('now','-3 year') AND %9<=date('now')");
                        output.push_back("%9>date('now','-5 year') AND %9<=date('now')");
                }
        }
        return output;
}

QStringList SKGRuleObject::getFromXML(SKGDocument* iDocument, const QString& iTable, const QString& iXML, bool iSQL, int iMode)
{
        QStringList output;
        if (iMode==2) {
                //Add markers
                output.push_back("UPDATE suboperation set i_tmp=1 WHERE id IN (SELECT i_SUBOPID FROM v_operation_consolidated WHERE #WC#)");
                output.push_back("UPDATE operation set i_tmp=1 WHERE id IN (SELECT id FROM v_operation_consolidated WHERE #WC#)");
        }

        QDomDocument doc("SKGML");
        doc.setContent(iXML);
        QDomElement root = doc.documentElement();

        QDomNode l = root.firstChild();
        while (!l.isNull()) {
                QDomElement elementl = l.toElement();
                if (!elementl.isNull()) {
                        QString lineDescription;

                        QDomNode n = elementl.firstChild();
                        bool parenthesisNeeded=false;
                        while (!n.isNull()) {
                                QDomElement element = n.toElement();
                                if (!element.isNull()) {
                                        //Build element description
                                        QString elementDescription;
                                        QString attribute=element.attribute ( "attribute");
                                        if (iSQL) {
                                                QString part=element.attribute ( "operator");
                                                if (part.contains("%1")) part=part.arg(SKGServices::stringToSqlString(element.attribute ( "value")));
                                                if (part.contains("%2")) part=part.arg(SKGServices::stringToSqlString(element.attribute ( "value2")));
                                                if (part.contains("%9")) part=part.arg(SKGServices::stringToSqlString(element.attribute ( "attribute")));

                                                elementDescription+=part;
                                        } else {
                                                attribute=iTable+'.'+attribute;
                                                if (iDocument) attribute=iDocument->getDisplay(attribute);

                                                elementDescription=attribute;
                                                elementDescription+=' ';

                                                QString part=SKGRuleObject::getDisplayForOperator(element.attribute ( "operator"),
                                                                SKGServices::stringToSqlString(element.attribute ( "value")),
                                                                SKGServices::stringToSqlString(element.attribute ( "value2")));

                                                elementDescription+=part;
                                        }

                                        if (elementDescription.contains("%9")) elementDescription=elementDescription.arg(attribute);

                                        //Add it to line description
                                        if (iSQL && iMode!=0) {
                                                if (iMode==1) {
                                                        output.push_back(elementDescription);
                                                } else if (iMode==2) {
                                                        QString sqlOrder;
                                                        if (attribute=="t_REALCATEGORY") {
                                                                elementDescription.replace(attribute, "t_fullname");

                                                                sqlOrder="UPDATE suboperation set r_category_id=(SELECT id FROM category WHERE ";
                                                                sqlOrder+=elementDescription;
                                                                sqlOrder+=") WHERE i_tmp=1";
                                                        } else if (attribute=="t_ACCOUNT") {
                                                                elementDescription.replace(attribute, "t_name");

                                                                sqlOrder="UPDATE operation set rd_account_id=(SELECT id FROM account WHERE ";
                                                                sqlOrder+=elementDescription;
                                                                sqlOrder+=") WHERE i_tmp=1";
                                                        } else if (attribute=="t_REFUND") {
                                                                elementDescription.replace(attribute, "t_name");

                                                                sqlOrder="UPDATE suboperation set r_refund_id=(SELECT id FROM refund WHERE ";
                                                                sqlOrder+=elementDescription;
                                                                sqlOrder+=") WHERE i_tmp=1";
                                                        } else {
                                                                sqlOrder="UPDATE operation set ";
                                                                sqlOrder+=elementDescription;
                                                                sqlOrder+=" WHERE i_tmp=1";
                                                        }

                                                        output.push_back(sqlOrder);
                                                }
                                        }

                                        if (!lineDescription.isEmpty()) {
                                                lineDescription+=(iMode==1 ? " , " : i18n(" and "));
                                                parenthesisNeeded=true;
                                        }
                                        lineDescription+=elementDescription;
                                }
                                n = n.nextSibling();
                        }

                        if (!(iMode!=0 && iSQL) && !lineDescription.isEmpty()) {
                                if (iMode==0 && parenthesisNeeded) lineDescription='('+lineDescription+')';
                                output.push_back(lineDescription);
                        }
                }
                l = l.nextSibling();
        }

        if (iMode==2) {
                //Remove markers
                output.push_back("UPDATE suboperation set i_tmp=0 WHERE i_tmp=1");
                output.push_back("UPDATE operation set i_tmp=0 WHERE i_tmp=1");
        }
        return output;
}

QString SKGRuleObject::getDescriptionFromXML(SKGDocument* iDocument, const QString& iTable, const QString& iXML, bool iSQL, int iMode)
{
        QString output;

        QStringList list=getFromXML(iDocument, iTable, iXML, iSQL, iMode);
        int nb=list.count();
        for (int i=0; i<nb; ++i) {
                output.append(list[i]);
                if (i<nb-1) output.append(iMode==1 ? " , " : (iSQL ? " OR " : i18n(" or ")));
        }
        return output;
}
#include "skgruleobject.moc"
