/***************************************************************************
 *   Copyright (C) 2008 by S. MANKOWSKI / G. DE BURE skrooge@miraks.com    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>  *
 ***************************************************************************/
/** @file
 * A skrooge plugin to search and process operations
 *
 * @author Stephane MANKOWSKI
 */
#include "skgsearchplugin.h"
#include "skgsearchpluginwidget.h"
#include "skgsearch_settings.h"
#include "skgtraces.h"

#include <kactioncollection.h>
#include <kstandardaction.h>

K_PLUGIN_FACTORY(SKGSearchPluginFactory, registerPlugin<SKGSearchPlugin>();)
K_EXPORT_PLUGIN(SKGSearchPluginFactory("skrooge_search", "skrooge_search"))

SKGSearchPlugin::SKGSearchPlugin(QObject* iParent, const QVariantList& /*iArg*/) : SKGInterfacePlugin(iParent)
{
        SKGTRACEIN(10, "SKGSearchPlugin::SKGSearchPlugin");
}

SKGSearchPlugin::~SKGSearchPlugin()
{
        SKGTRACEIN(10, "SKGSearchPlugin::~SKGSearchPlugin");
        currentBankDocument=NULL;
}

void SKGSearchPlugin::setupActions(SKGDocument* iDocument, const QStringList& iArgument)
{
        SKGTRACEIN(10, "SKGSearchPlugin::setupActions");
        Q_UNUSED(iArgument);

        currentBankDocument=iDocument;


        setComponentData( SKGSearchPluginFactory::componentData() );
        setXMLFile("skrooge_search.rc");

        //Create yours actions here
}

void SKGSearchPlugin::refresh()
{
        SKGTRACEIN(10, "SKGSearchPlugin::refresh");
}

void SKGSearchPlugin::close()
{
        SKGTRACEIN(10, "SKGSearchPlugin::close");
}

SKGTabWidget* SKGSearchPlugin::getWidget()
{
        SKGTRACEIN(10, "SKGSearchPlugin::getWidget");
        return new SKGSearchPluginWidget(currentBankDocument);
}

QWidget* SKGSearchPlugin::getPreferenceWidget()
{
        SKGTRACEIN(10, "SKGSearchPlugin::getPreferenceWidget");
        /*QWidget* widget=new QWidget();
        ui.setupUi(widget);*/

        return NULL;
}

KConfigSkeleton* SKGSearchPlugin::getPreferenceSkeleton()
{
        return skgsearch_settings::self();
}

QString SKGSearchPlugin::title() const
{
        return i18n("Search and process");
}

QString SKGSearchPlugin::icon() const
{
        return "edit-find";
}

QString SKGSearchPlugin::statusTip () const
{
        return title();
}

QString SKGSearchPlugin::toolTip () const
{
        return title();
}


int SKGSearchPlugin::getOrder() const
{
        return 35;
}

QStringList SKGSearchPlugin::tips() const
{
        QStringList output;
        output.push_back(i18n("<p>... skrooge can search and automatically process some operations.</p>"));
        return output;
}

bool SKGSearchPlugin::isInContext() const
{
        return true;
}

#include "skgsearchplugin.moc"
